﻿using IRaCIS.Application.Contracts;
using IRaCIS.Core.Domain.Share;
using IRaCIS.Core.Infrastructure;
using System.Text.RegularExpressions;

using Microsoft.AspNetCore.Mvc;
using Panda.DynamicWebApi.Attributes;
using Microsoft.AspNetCore.Authorization;

namespace IRaCIS.Application.Services
{
    [ApiExplorerSettings(GroupName = "Management")]
    public class UserService : BaseService, IUserService
    {
        private readonly IRepository<User> _userRepository;
        private readonly IMailVerificationService _mailVerificationService;
        private readonly IRepository<VerificationCode> _verificationCodeRepository;
        private readonly IRepository<Doctor> _doctorRepository;
        private readonly IRepository<TrialUser> _userTrialRepository;
        public UserService(IRepository<User> userRepository,

            IMailVerificationService mailVerificationService,
            IRepository<VerificationCode> verificationCodeRepository,
            IRepository<Doctor> doctorRepository,
            IRepository<TrialUser> userTrialRepository

         )
        {
            _userRepository = userRepository;
            _mailVerificationService = mailVerificationService;
            _verificationCodeRepository = verificationCodeRepository;
            _doctorRepository = doctorRepository;
            _userTrialRepository = userTrialRepository;
        }

        /// <summary>发送验证码  修改邮箱（已经登陆修改） New </summary>

        [HttpGet("{email}")]
        public async Task<IResponseOutput> SendVerificationCode(string email)
        {

            //检查手机或者邮箱是否有效
            if (!Regex.IsMatch(email, @"^[a-zA-Z0-9_-]+@[a-zA-Z0-9_-]+(\.[a-zA-Z0-9_-]+)+$"))
            {

                return ResponseOutput.NotOk("Please input a legal email");

            }


            //验证码 6位
            int verificationCode = new Random().Next(100000, 1000000);

            await _mailVerificationService.SendMailEditEmail(_userInfo.Id, _userInfo.RealName, email, verificationCode);

            return ResponseOutput.Ok();

        }


        [HttpPut("{newEmail}/{verificationCode}")]
        public async Task<IResponseOutput> SetNewEmail(string newEmail, string verificationCode)
        {


            var verificationRecord = await _verificationCodeRepository
                .FirstOrDefaultAsync(t => t.UserId == _userInfo.Id && t.Code == verificationCode && t.CodeType == 0);

            //检查数据库是否存在该验证码
            if (verificationRecord == null)
            {

                return ResponseOutput.NotOk("Verification code error");

            }
            else
            {
                //检查验证码是否失效
                if (verificationRecord.ExpirationTime < DateTime.Now)
                {
                    return ResponseOutput.NotOk("The verification code has expired");

                }
                else //验证码正确 并且 没有超时 
                {
                    //更新密码
                    //var pwd = MD5Helper.Md5(newPwd);
                    //var count = _doctorRepository.Update<Doctor>().Where(t => t.Id == doctor.Id).Set(d => d.Password == pwd).ExecuteAffrows();


                    if (await _userRepository.AnyAsync(t => (t.EMail == newEmail && t.UserTypeId == _userInfo.UserTypeId && t.Id != _userInfo.Id)))
                    {
                        return ResponseOutput.NotOk("The mailbox  for this user type already exists");
                    }

                    await _userRepository.UpdatePartialNowNoQueryAsync(_userInfo.Id, u => new User()
                    {
                        EMail = newEmail
                    });

                    //删除验证码历史记录
                    await _verificationCodeRepository.BatchDeleteNoTrackingAsync(t => t.UserId == _userInfo.Id && t.CodeType == 0);

                    return ResponseOutput.Ok();

                }
            }
        }


        [HttpPut("{newPhone}")]
        public async Task<IResponseOutput> SetNewPhone(string newPhone)
        {


            await _userRepository.UpdatePartialNowNoQueryAsync(_userInfo.Id, u => new User()
            {
                Phone = newPhone
            });

            return ResponseOutput.Ok();
        }


        [HttpPut("{newUserName}")]
        public async Task<IResponseOutput> SetNewUserName(string newUserName)
        {

            if (await _userRepository.AnyAsync(t => t.UserName == newUserName && t.Id != _userInfo.Id))
            {
                return ResponseOutput.NotOk("UserId  already exists");
            }
            await _userRepository.UpdatePartialNowNoQueryAsync(_userInfo.Id, u => new User()
            {
                UserName = newUserName
            });

            return ResponseOutput.Ok();
        }


        /// <summary>
        /// 重置密码为 默认密码
        /// </summary>
        /// <param name="userId"></param>
        /// <returns></returns>
        [HttpGet("{userId:guid}")]

        public async Task<IResponseOutput> ResetPassword(Guid userId)
        {
            await _userRepository.UpdatePartialNowNoQueryAsync(userId, u => new User()
            {
                Password = MD5Helper.Md5(StaticData.DefaultPassword),
                IsFirstAdd = true
            });

            return ResponseOutput.Ok();
        }



        /// <summary>
        ///  重置密码发邮件  （未登陆修改）
        /// </summary>
        /// <param name="email"></param>
        /// <returns></returns>
        [AllowAnonymous]
        [HttpGet("{email}")]
        public async Task<IResponseOutput> AnonymousSendVerificationCode(string email)
        {

            //检查手机或者邮箱是否有效
            if (!Regex.IsMatch(email, @"^[a-zA-Z0-9_-]+@[a-zA-Z0-9_-]+(\.[a-zA-Z0-9_-]+)+$"))
            {

                return ResponseOutput.NotOk("Please input a legal email");

            }

            ////查找改邮箱或者手机的用户
            var exist = await _userRepository.AnyAsync(t => t.EMail == email);

            if (!exist)
            {
                return ResponseOutput.NotOk("Email  not  correct");

            }


            //验证码 6位
            int verificationCode = new Random().Next(100000, 1000000);

            await _mailVerificationService.AnolymousSendEmailForResetAccount(email, verificationCode);

            return ResponseOutput.Ok();

        }

        /// <summary>
        /// 验证验证码，没问题就返回用户所有的账户
        /// </summary>
        /// <param name="email"></param>
        /// <param name="verifyCode"></param>
        /// <returns></returns>
        /// <exception cref="BusinessValidationFailedException"></exception>
        [AllowAnonymous]
        [HttpGet("{email}/{verifyCode}")]
        public async Task<List<UserAccountDto>> VerifyAnonymousVerifyCode(string email, string verifyCode)
        {
            var verificationRecord = await _verificationCodeRepository
             .Where(t => t.UserId == Guid.Empty && t.Code == verifyCode && t.CodeType == VerifyType.Email && t.EmailOrPhone == email).OrderByDescending(t => t.CreateTime).FirstOrDefaultAsync();

            //检查数据库是否存在该验证码
            if (verificationRecord == null)
            {

                throw new BusinessValidationFailedException("Verification code error");
            }
            else
            {
                //检查验证码是否失效
                if (verificationRecord.ExpirationTime < DateTime.Now)
                {

                    throw new BusinessValidationFailedException("The verification code has expired");
                }
                else //验证码正确 并且 没有超时 
                {

                    //删除验证码历史记录
                    await _verificationCodeRepository.BatchDeleteNoTrackingAsync(t => t.Id == verificationRecord.Id);
                }
            }

            var list = await _userRepository.Where(t => t.EMail == email).Select(t => new UserAccountDto() { UserId = t.Id, UserName = t.UserName, UserRealName = t.FullName, UserType = t.UserTypeRole.UserTypeShortName }).ToListAsync();



            return list;
        }


        /// <summary>
        ///  （未登陆） 设置新密码
        /// </summary>
        /// <param name="userId"></param>
        /// <param name="newPwd"></param>
        /// <returns></returns>
        [AllowAnonymous]
        [HttpGet("{userId:guid}/{newPwd}")]
        public async Task<IResponseOutput> AnonymousSetPassword(Guid userId, string newPwd)
        {


            var dbUser = await _userRepository.FirstOrDefaultAsync(t => t.Id == userId);
            if (dbUser != null)
            {
                if (dbUser.Password == newPwd)
                {
                    return ResponseOutput.NotOk("password not change");
                }

                var success = await _userRepository.BatchUpdateNoTrackingAsync(t => t.Id == userId, u => new User()
                {
                    Password = newPwd,
                    IsFirstAdd = false
                });

                return ResponseOutput.Result(success);
            }
            return ResponseOutput.NotOk("UserId 传递有误");
        }

        /// <summary>
        /// 修改密码,当前支持旧密码修改密码
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        [UnitOfWork]
        public async Task<IResponseOutput> ModifyPassword(EditPasswordCommand editPwModel)
        {

            //验证旧密码OK
            var dbUser = await _userRepository.FirstOrDefaultAsync(t => t.Id == _userInfo.Id && t.Password == editPwModel.OldPassWord);

            if (dbUser != null)
            {
                if (dbUser.Password == editPwModel.NewPassWord)
                {
                    return ResponseOutput.NotOk("password not change");
                }

                if (!string.IsNullOrEmpty(editPwModel.NewUserName))
                {
                    if (await _userRepository.AnyAsync(t => t.UserName == editPwModel.NewUserName && t.Id != _userInfo.Id))
                    {
                        return ResponseOutput.NotOk("UserId  already exists");
                    }

                    await _userRepository.BatchUpdateNoTrackingAsync(t => t.Id == _userInfo.Id, u => new User()
                    {
                        UserName = editPwModel.NewUserName,
                    });

                }

                var success = await _userRepository.BatchUpdateNoTrackingAsync(t => t.Id == _userInfo.Id, u => new User()
                {
                    Password = editPwModel.NewPassWord,
                    IsFirstAdd = false
                });



                return ResponseOutput.Result(success);
            }

            //医生密码
            if (await _doctorRepository.AnyAsync(t => t.Id == _userInfo.Id && t.Password == editPwModel.OldPassWord))
            {
                var success = await _doctorRepository.BatchUpdateNoTrackingAsync(t => t.Id == _userInfo.Id, u => new Doctor()
                {

                    Password = editPwModel.NewPassWord
                });

                return ResponseOutput.Result(success);
            }

            return ResponseOutput.NotOk("Old password is wrong.");

        }



        /// <summary>
        /// 获取用户列表
        /// </summary>
        /// <param name="param"></param>
        /// <returns></returns>
        [HttpPost]
        public async Task<PageOutput<UserListDTO>> GetUserList(UserListQueryDTO param)
        {
            var userQueryable = _userRepository.Where(x => x.UserTypeEnum != UserTypeEnum.SuperAdmin)
                  .WhereIf(!string.IsNullOrWhiteSpace(param.UserName), t => t.UserName.Contains(param.UserName) || (t.FullName).Contains(param.UserName))
                  .WhereIf(!string.IsNullOrWhiteSpace(param.Phone), t => t.Phone.Contains(param.Phone))
                  .WhereIf(!string.IsNullOrWhiteSpace(param.OrganizationName), t => t.OrganizationName.Contains(param.OrganizationName))
                  .WhereIf(param.UserType != null, t => t.UserTypeId == param.UserType)
                  .WhereIf(param.UserState != null, t => t.Status == param.UserState)
                  .ProjectTo<UserListDTO>(_mapper.ConfigurationProvider);

            return await userQueryable.ToPagedListAsync(param.PageIndex, param.PageSize, param.SortField == string.Empty ? "UserName" : param.SortField, param.Asc);


        }

        /// <summary>
        /// 根据用户Id获取用户详细信息[New]
        /// </summary>
        /// <param name="id"></param>
        /// <returns></returns>
        [HttpGet("{id:guid}")]
        public async Task<UserDetailDTO> GetUser(Guid id)
        {
            var userQuery = _userRepository.Where(t => t.Id == id).ProjectTo<UserDetailDTO>(_mapper.ConfigurationProvider);
            return await (userQuery.FirstOrDefaultAsync()).IfNullThrowException();
        }

        /// <summary>
        /// 添加用户
        /// </summary>
        /// <param name="userAddModel"></param>
        /// <returns></returns>
        public async Task<IResponseOutput<UserAddedReturnDTO>> AddUser(UserCommand userAddModel)
        {
            if (await _userRepository.AnyAsync(t => t.UserName == userAddModel.UserName || (t.EMail == userAddModel.EMail && t.UserTypeId == userAddModel.UserTypeId)))
            {
                return ResponseOutput.NotOk(" UserId   or The mailbox  for this user type already exists", new UserAddedReturnDTO());
            }

            var saveItem = _mapper.Map<User>(userAddModel);

            saveItem.Code = await _userRepository.Select(t => t.Code).DefaultIfEmpty().MaxAsync() + 1;

            saveItem.UserCode = AppSettings.UserCodePrefix + saveItem.Code.ToString("D4");

            if (saveItem.IsZhiZhun)
            {
                saveItem.OrganizationName = "Zhizhun";
            }

            //验证码 6位
            int verificationCode = new Random().Next(100000, 1000000);

            saveItem.Password = MD5Helper.Md5("123456");

            await _userRepository.AddAsync(saveItem);

            var success = await _userRepository.SaveChangesAsync();

            return ResponseOutput.Result(success, new UserAddedReturnDTO { Id = saveItem.Id, UserCode = saveItem.UserCode, VerificationCode = verificationCode });

        }

        /// <summary>
        /// 更新用户
        /// </summary>
        /// <param name="model"></param>
        /// <returns></returns>
        public async Task<IResponseOutput> UpdateUser(UserCommand model)
        {

            // 判断当前用户名是否已经存在
            if (await _userRepository.AnyAsync(t => (t.UserName == model.UserName && t.Id != model.Id) || (t.EMail == model.EMail && t.UserTypeId == model.UserTypeId && t.Id != model.Id)))
            {
                return ResponseOutput.NotOk("UserId   or The mailbox  for this user type already exists");
            }

            var user = await _userRepository.FirstOrDefaultAsync(t => t.Id == model.Id);

            if (user == null) return Null404NotFound(user);

            _mapper.Map(model, user);

            if (user.IsZhiZhun)
            {
                user.OrganizationName = "Zhizhun";
            }
            var success = await _userRepository.SaveChangesAsync();

            return ResponseOutput.Result(success);

        }

        /// <summary>
        /// 删除用户
        /// </summary>
        /// <param name="userId"></param>
        /// <returns></returns>
        [HttpDelete("{userId:guid}")]
        public async Task<IResponseOutput> DeleteUser(Guid userId)
        {
            if (await _userTrialRepository.AnyAsync(t => t.Id == userId))
            {
                return ResponseOutput.NotOk("This user has participated in the trial and couldn't be deleted");
            }

            var success = await _userRepository.BatchDeleteNoTrackingAsync(t => t.Id == userId);

            return ResponseOutput.Result(success);
        }

        /// <summary>
        /// 禁用或者启用账户
        /// </summary>
        /// <param name="userId"></param>
        /// <param name="state"></param>
        /// <returns></returns>

        [HttpPost("{userId:guid}/{state:int}")]
        public async Task<IResponseOutput> UpdateUserState(Guid userId, UserStateEnum state)
        {
            var success = await _userRepository.BatchUpdateNoTrackingAsync(u => u.Id == userId, t => new User
            {
                Status = state
            });
            return ResponseOutput.Result(success);
        }




        /// <summary>
        /// 用户登陆
        /// </summary>
        /// <param name="userName"></param>
        /// <param name="password"></param>
        /// <returns></returns>
        [NonDynamicMethod]
        public async Task<IResponseOutput<LoginReturnDTO>> Login(string userName, string password)
        {
            var userLoginReturnModel = new LoginReturnDTO();


            var loginUser = await _userRepository.Where(u => u.UserName.Equals(userName) && u.Password == password).ProjectTo<UserBasicInfo>(_mapper.ConfigurationProvider).FirstOrDefaultAsync();

            if (loginUser == null)
            {
                //此处下面 代码 为了支持医生也能登录 而且前端不加选择到底是管理用户 还是医生用户  奇怪的需求 无法理解

                var loginDoctor = await _doctorRepository.Where(u => u.Phone == userName && u.Password == password).ProjectTo<UserBasicInfo>(_mapper.ConfigurationProvider).FirstOrDefaultAsync();

                if (loginDoctor == null)
                {
                    return ResponseOutput.NotOk("Please check the user name or password.", new LoginReturnDTO());

                }

                userLoginReturnModel.BasicInfo = loginDoctor;


                return ResponseOutput.Ok(userLoginReturnModel);

            }

            if (loginUser.Status == 0)
            {
                return ResponseOutput.NotOk("The user has been disabled!", new LoginReturnDTO());
            }

            userLoginReturnModel.BasicInfo = loginUser;


            return ResponseOutput.Ok(userLoginReturnModel);

        }

    }
}
